@@ -30,6 +30,7 @@ protected:
 	virtual int List(CServerPath path = CServerPath(), wxString subDir = _T(""), bool refresh = false);
 	int ListParseResponse();
 	int ListSend(int prevResult = FZ_REPLY_OK);
+	int ListCheckTimezoneDetection(CDirectoryListing& listing);
 
 	int ChangeDir(CServerPath path = CServerPath(), wxString subDir = _T(""));
 	int ChangeDirParseResponse();
@@ -46,7 +47,7 @@ protected:
 	int RawCommandSend();
 	int RawCommandParseResponse();
 
-	virtual int Delete(const CServerPath& path, const wxString& file);
+	virtual int Delete(const CServerPath& path, const std::list<wxString>& files);
 	int DeleteSend(int prevResult = FZ_REPLY_OK);
 	int DeleteParseResponse();
 	
@@ -102,8 +103,13 @@ protected:
 	// Checks if listing2 is a subset of listing1. Compares only filenames.
 	bool CheckInclusion(const CDirectoryListing& listing1, const CDirectoryListing& listing2);
 
+	void StartKeepaliveTimer();
+
+	bool GetLoginSequence(const CServer& server);
+
 	wxString m_Response;
 	wxString m_MultilineResponseCode;
+	std::list<wxString> m_MultilineResponseLines;
 
 	CTransferSocket *m_pTransferSocket;
 
@@ -126,8 +132,15 @@ protected:
 
 	int m_pendingTransferEndEvents;
 
+	// Used by keepalive code so that we're not using keep alive
+	// till the end of time. Stop after a couple of minutes.
+	wxDateTime m_lastCommandCompletionTime;
+
+	wxTimer m_idleTimer;
+
 	DECLARE_EVENT_TABLE();
 	void OnExternalIPAddress(fzExternalIPResolveEvent& event);
+	void OnIdleTimer(wxTimerEvent& event);
 };
 
 class CIOThread;
