@@ -117,19 +117,43 @@ bool CServerPath::SetPath(wxString &newPath, bool isFile)
 			m_bEmpty = true;
 			return false;
 		}
-		path.RemoveLast();
-		if (pos)
-			m_prefix = path.Left(pos);
-		path = path.Mid(pos + 1);
-		pos = path.Find(_T("."));
-		while (pos != -1)
+		else
 		{
-			m_Segments.push_back(path.Left(pos));
+			path.RemoveLast();
+			if (pos)
+				m_prefix = path.Left(pos);
 			path = path.Mid(pos + 1);
 			pos = path.Find(_T("."));
+			bool append = false;
+			while (pos != -1)
+			{
+				bool wasAppend = append;
+				wxString segment = path.Left(pos);
+				if (segment.Last() == '^')
+				{
+					append = true;
+					segment.RemoveLast();
+					segment += _T(".");
+				}
+				else
+					append = false;
+
+				if (wasAppend)
+					m_Segments.back() += segment;
+				else
+					m_Segments.push_back(segment);					
+
+				path = path.Mid(pos + 1);
+				pos = path.Find(_T("."));
+			}
+			if (path != _T(""))
+			{
+				if (append)
+					m_Segments.back() += path;
+				else
+					m_Segments.push_back(path);
+			}
 		}
-		if (path != _T(""))
-			m_Segments.push_back(path);
 		break;
 	case MVS:
 		{
@@ -287,7 +311,11 @@ wxString CServerPath::GetPath() const
 		{
 			path = m_prefix + _T("[");
 			for (tConstSegmentIter iter = m_Segments.begin(); iter != m_Segments.end(); iter++)
-				path += *iter + _T(".");
+			{
+				wxString segment = *iter;
+				segment.Replace(_T("."), _T("^."));
+				path += segment + _T(".");
+			}
 			path.RemoveLast();
 			path += _T("]");
 		}
@@ -424,27 +452,26 @@ bool CServerPath::SetSafePath(wxString path)
 	if (len)
 	{
 		m_prefix = path.Left(len);
-		path = path.Mid(len);
+		path = path.Mid(len + 1);
 	}
 
 	while (path != _T(""))
 	{
 		pos = path.Find(' ');
-		if (pos < 1)
+		if (pos == -1)
 			return false;
+		if (!pos)
+		{
+			path = path.Mid(1);
+			continue;
+		}
 		
-		if (!path.Left(pos).ToULong(&len))
+		if (!path.Left(pos).ToULong(&len) || !len)
 			return false;
 		path = path.Mid(pos + 1);
 		if (path.Length() < len)
 			return false;
 
-		if (!len)
-			return false;
-
-		if (path.Length() < len)
-			return false;
-
 		m_Segments.push_back(path.Left(len));
 		path = path.Mid(len + 1);
 	}
@@ -584,14 +611,35 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 				m_Segments.clear();
 			}
 			int pos = dir.Find(_T("."));
+			bool append = false;
 			while (pos != -1)
 			{
-				m_Segments.push_back(dir.Left(pos));
+				bool wasAppend = append;
+				wxString segment = dir.Left(pos);
+				if (segment.Last() == '^')
+				{
+					append = true;
+					segment.RemoveLast();
+					segment += _T(".");
+				}
+				else
+					append = false;
+
+				if (wasAppend)
+					m_Segments.back() += segment;
+				else
+					m_Segments.push_back(segment);					
+
 				dir = dir.Mid(pos + 1);
 				pos = dir.Find(_T("."));
 			}
 			if (dir != _T(""))
-				m_Segments.push_back(dir);
+			{
+				if (append)
+					m_Segments.back() += dir;
+				else
+					m_Segments.push_back(dir);
+			}
 		}
 		break;
 	case DOS:
@@ -855,6 +903,43 @@ bool CServerPath::operator!=(const CServerPath &op) const
 	return !(*this == op);
 }
 
+bool CServerPath::operator<(const CServerPath &op) const
+{
+	if (m_bEmpty)
+	{
+		if (!op.m_bEmpty)
+			return false;
+	}
+	else if (op.m_bEmpty)
+		return true;
+
+	const int cmp = m_prefix.Cmp(op.m_prefix);
+	if (cmp < 0)
+		return true;
+	if (cmp > 0)
+		return false;
+	
+	if (m_type > op.m_type)
+		return false;
+	else if (m_type < op.m_type)
+		return true;
+
+	std::list<wxString>::const_iterator iter1, iter2;
+	for (iter1 = m_Segments.begin(), iter2 = op.m_Segments.begin(); iter1 != op.m_Segments.end(); iter1++, iter2++)
+	{
+		if (iter2 == m_Segments.end())
+			return false;
+
+		const int cmp = iter1->Cmp(*iter2);
+		if (cmp < 0)
+			return true;
+		if (cmp > 0)
+			return false;
+	}
+
+	return iter2 != op.m_Segments.end();
+}
+
 wxString CServerPath::FormatFilename(const wxString &filename, bool omitPath /*=false*/) const
 {
 	if (m_bEmpty)
@@ -968,3 +1053,14 @@ CServerPath CServerPath::GetCommonParent(const CServerPath& path) const
 
 	return parent;
 }
+
+wxString CServerPath::FormatSubdir(const wxString &subdir) const
+{
+	if (m_type != VMS)
+		return subdir;
+
+	wxString res = subdir;
+	res.Replace(_T("."), _T("^."));
+
+	return res;
+}
