@@ -55,6 +55,8 @@ public:
 		wxDateTime lastModified;
 	};
 
+	void InitDateFormat();
+
 protected:
 	bool IsItemValid(unsigned int item) const;
 	t_fileData *GetData(unsigned int item);
@@ -90,6 +92,8 @@ protected:
 
 	bool m_hasParent;
 
+	wxString m_dateFormat;
+
 	// Event handlers
 	DECLARE_EVENT_TABLE();
 	void OnItemActivated(wxListEvent& event);
