@@ -94,6 +94,7 @@ protected:
 	void OnSize(wxSizeEvent& event);
 	void OnViewSplitterPosChanged(wxSplitterEvent& event);
 	void OnMenuHandler(wxCommandEvent& event);
+	void OnMenuOpenHandler(wxMenuEvent& event);
 	void OnEngineEvent(wxEvent& event);
 	void OnUpdateToolbarDisconnect(wxUpdateUIEvent& event);
 	void OnDisconnect(wxCommandEvent& event);
@@ -130,6 +131,13 @@ protected:
 	void OnNavigationKeyEvent(wxNavigationKeyEvent& event);
 	void OnGetFocus(wxFocusEvent& event);
 	void OnChar(wxKeyEvent& event);
+	void OnMoveEvent(wxMoveEvent& event);
+	void OnActivate(wxActivateEvent& event);
+#ifdef __WXMSW__
+	void OnSizePost(wxCommandEvent& event);
+
+	bool m_pendingPostSizing;
+#endif
 
 	float m_ViewSplitterSashPos;
 	bool m_bInitDone;
