@@ -4,6 +4,7 @@
 #include "xmlfunctions.h"
 #include "filezillaapp.h"
 #include <wx/tokenzr.h>
+#include "ipcmutex.h"
 
 #ifdef _DEBUG
 #define new DEBUG_NEW
@@ -37,7 +38,7 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Last resolved IP", string, _T("") },
 	{ "No external ip on local conn", number, _T("1") },
 	{ "Pasv reply fallback mode", number, _T("0") },
-	{ "Timeout", number, _T("15") },
+	{ "Timeout", number, _T("20") },
 	{ "Logging Debug Level", number, _T("0") },
 	{ "Logging Raw Listing", number, _T("0") },
 	{ "fzsftp executable", string, _T("") },
@@ -48,10 +49,19 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Speedlimit outbound", number, _T("0") },
 	{ "Speedlimit burst tolerance", number, _T("0") },
 	{ "View hidden files", number, _T("0") },
+	{ "Preserve timestamps", number, _T("0") },
+	{ "Socket recv buffer size", number, _T("131072") }, // Make it large enough by default 
+														 // to enable a large TCP window scale
+	{ "Socket send buffer size", number, _T("131072") },
+	{ "FTP Keep-alive commands", number, _T("0") },
+	{ "FTP Proxy type", number, _T("0") },
+	{ "FTP Proxy host", string, _T("") },
+	{ "FTP Proxy user", string, _T("") },
+	{ "FTP Proxy password", string, _T("") },
+	{ "FTP Proxy login sequence", string, _T("") },
 
 	// Interface settings
 	{ "Number of Transfers", number, _T("2") },
-	{ "Transfer Retry Count", number, _T("5") },
 	{ "Ascii Binary mode", number, _T("0") },
 	{ "Auto Ascii files", string, _T("am|asp|bat|c|cfm|cgi|conf|cpp|css|dhtml|diz|h|hpp|htm|html|in|inc|js|m4|mak|nfo|nsi|pas|patch|php|phtml|pl|po|py|qmail|sh|shtml|sql|tcl|tpl|txt|vbs|xml|xrc") },
 	{ "Auto Ascii no extension", number, _T("1") },
@@ -84,7 +94,17 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Remote filelist colwidths", string, _T("") },
 	{ "Window position and size", string, _T("") },
 	{ "Local filelist sortorder", string, _T("") },
-	{ "Remote filelist sortorder", string, _T("") }
+	{ "Remote filelist sortorder", string, _T("") },
+	{ "Time Format", string, _T("") },
+	{ "Date Format", string, _T("") },
+	{ "Show message log", number, _T("1") },
+	{ "Show queue", number, _T("1") },
+	{ "Size format", number, _T("0") },
+	{ "Size thousands separator", number, _T("1") },
+	{ "Default editor", string, _T("") },
+	{ "Always use default editor", number, _T("0") },
+	{ "Inherit system associations", number, _T("1") },
+	{ "Custom file associations", string, _T("") },
 };
 
 COptions::COptions()
@@ -96,6 +116,7 @@ COptions::COptions()
 	for (unsigned int i = 0; i < OPTIONS_NUM; i++)
 		m_optionsCache[i].cached = false;
 
+	CInterProcessMutex mutex(MUTEX_OPTIONS);
 	m_pXmlFile = new CXmlFile(_T("filezilla"));
 	if (!m_pXmlFile->Load())
 	{
@@ -170,7 +191,10 @@ bool COptions::SetOption(unsigned int nID, int value)
 		return false;
 
 	if (m_pXmlFile)
+	{
+		CInterProcessMutex mutex(MUTEX_OPTIONS);
 		m_pXmlFile->Save();
+	}
 
 	return true;
 }
@@ -200,7 +224,10 @@ bool COptions::SetOption(unsigned int nID, wxString value)
 		return false;
 	
 	if (m_pXmlFile)
+	{
+		CInterProcessMutex mutex(MUTEX_OPTIONS);
 		m_pXmlFile->Save();
+	}
 
 	return true;
 }
@@ -401,6 +428,14 @@ int COptions::Validate(unsigned int nID, int value)
 		if (value < 0 || value > 2)
 			value = 0;
 		break;
+	case OPTION_SOCKET_BUFFERSIZE_RECV:
+		if (value != -1 && (value < 4096 || value > 4096 * 1024))
+			value = -1;
+		break;
+	case OPTION_SOCKET_BUFFERSIZE_SEND:
+		if (value != -1 && (value < 4096 || value > 4096 * 1024))
+			value = 131072;
+		break;
 	}
 	return value;
 }
@@ -456,6 +491,7 @@ void COptions::SetServer(wxString path, const CServer& server)
 
 	::SetServer(element, server);
 
+	CInterProcessMutex mutex(MUTEX_OPTIONS);
 	m_pXmlFile->Save();
 }
 
@@ -556,7 +592,10 @@ void COptions::Import(TiXmlElement* pElement)
 	}
 
 	if (m_pXmlFile)
+	{
+		CInterProcessMutex mutex(MUTEX_OPTIONS);
 		m_pXmlFile->Save();
+	}
 }
 
 void COptions::SaveColumnWidths(const wxListCtrl* const pListCtrl, unsigned int optionId)
