@@ -19,6 +19,10 @@
 #include "queueview_failed.h"
 #include "queueview_successful.h"
 #include "commandqueue.h"
+#include <wx/utils.h>
+#include <wx/progdlg.h>
+#include <wx/sound.h>
+#include "edithandler.h"
 
 #ifdef _DEBUG
 #define new DEBUG_NEW
@@ -155,6 +159,14 @@ EVT_MENU(XRCID("ID_PROCESSQUEUE"), CQueueView::OnProcessQueue)
 EVT_MENU(XRCID("ID_REMOVEALL"), CQueueView::OnStopAndClear)
 EVT_MENU(XRCID("ID_REMOVE"), CQueueView::OnRemoveSelected)
 EVT_MENU(XRCID("ID_DEFAULT_FILEEXISTSACTION"), CQueueView::OnSetDefaultFileExistsAction)
+EVT_MENU(XRCID("ID_ACTIONAFTER_DISABLE"), CQueueView::OnActionAfter)
+EVT_MENU(XRCID("ID_ACTIONAFTER_CLOSE"), CQueueView::OnActionAfter)
+EVT_MENU(XRCID("ID_ACTIONAFTER_DISCONNECT"), CQueueView::OnActionAfter)
+EVT_MENU(XRCID("ID_ACTIONAFTER_RUNCOMMAND"), CQueueView::OnActionAfter)
+EVT_MENU(XRCID("ID_ACTIONAFTER_SHOWMESSAGE"), CQueueView::OnActionAfter)
+EVT_MENU(XRCID("ID_ACTIONAFTER_PLAYSOUND"), CQueueView::OnActionAfter)
+EVT_MENU(XRCID("ID_ACTIONAFTER_REBOOT"), CQueueView::OnActionAfter)
+EVT_MENU(XRCID("ID_ACTIONAFTER_SHUTDOWN"), CQueueView::OnActionAfter)
 
 EVT_COMMAND(wxID_ANY, fzEVT_ASKFORPASSWORD, CQueueView::OnAskPassword)
 
@@ -412,6 +424,13 @@ CQueueView::CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsync
 	m_totalQueueSize = 0;
 	m_filesWithUnknownSize = 0;
 
+	m_actionAfterState = ActionAfterState_Disabled;
+#ifdef __WXMSW__
+	m_actionAfterWarnDialog = 0;
+	m_actionAfterTimer = 0;
+	m_actionAfterTimerId = -1;
+#endif
+
 	CreateColumns(_("Status"));
 
 	//Initialize the engine data
@@ -424,22 +443,6 @@ CQueueView::CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsync
 	pData->m_idleDisconnectTimer = 0;
 	m_engineData.push_back(pData);
 
-	int engineCount = COptions::Get()->GetOptionVal(OPTION_NUMTRANSFERS);
-	for (int i = 0; i < engineCount; i++)
-	{
-		t_EngineData *pData = new t_EngineData;
-		pData->active = false;
-		pData->pItem = 0;
-		pData->pStatusLineCtrl = 0;
-		pData->state = t_EngineData::none;
-		pData->m_idleDisconnectTimer = 0;
-
-		pData->pEngine = new CFileZillaEngine();
-		pData->pEngine->Init(this, COptions::Get());
-
-		m_engineData.push_back(pData);
-	}
-
 	SettingsChanged();
 
 	SetDropTarget(new CQueueViewDropTarget(this));
@@ -471,7 +474,7 @@ CQueueView::~CQueueView()
 
 bool CQueueView::QueueFile(const bool queueOnly, const bool download, const wxString& localFile,
 						   const wxString& remoteFile, const CServerPath& remotePath,
-						   const CServer& server, const wxLongLong size)
+						   const CServer& server, const wxLongLong size, bool edit /*=false*/)
 {
 	CServerItem* pServerItem = CreateServerItem(server);
 
@@ -482,11 +485,13 @@ bool CQueueView::QueueFile(const bool queueOnly, const bool download, const wxSt
 			fileItem = new CFolderItem(pServerItem, queueOnly, localFile);
 		else
 			fileItem = new CFolderItem(pServerItem, queueOnly, remotePath, remoteFile);
+		wxASSERT(!edit);
 	}
 	else
 	{
 		fileItem = new CFileItem(pServerItem, queueOnly, download, localFile, remoteFile, remotePath, size);
 		fileItem->m_transferSettings.binary = ShouldUseBinaryMode(download ? remoteFile : wxFileName(localFile).GetFullName());
+		fileItem->m_edit = edit;
 	}
 
 	InsertItem(pServerItem, fileItem);
@@ -730,6 +735,8 @@ bool CQueueView::TryStartNextTransfer()
 		{
 			wxFileName fn(newFileItem->GetLocalFile(), _T(""));
 			wxFileName::Mkdir(fn.GetPath(), 0777, wxPATH_MKDIR_FULL);
+			CState* const pState = m_pMainFrame->GetState();
+			pState->RefreshLocalFile(fn.GetFullPath());
 			if (RemoveItem(newFileItem, true))
 			{
 				// Server got deleted. Unfortunately we have to start over now
@@ -1033,6 +1040,14 @@ void CQueueView::ResetEngine(t_EngineData& data, const enum ResetReason reason)
 			const CFileItem* const pFileItem = (CFileItem*)data.pItem;
 			if (pFileItem->Download())
 				m_pMainFrame->GetState()->RefreshLocalFile(pFileItem->GetLocalFile());
+
+			if (pFileItem->m_edit && reason != retry && reason != reset)
+			{
+				CEditHandler* pEditHandler = CEditHandler::Get();
+				wxASSERT(pEditHandler);
+				wxFileName fn(pFileItem->GetLocalFile());
+				pEditHandler->FinishTransfer(reason == success, fn.GetFullName());
+			}
 		}
 
 		wxASSERT(data.pItem->IsActive());
@@ -1360,6 +1375,16 @@ bool CQueueView::Quit()
 {
 	m_quit = true;
 
+#ifdef __WXMSW__
+	if (m_actionAfterWarnDialog)
+	{
+		m_actionAfterWarnDialog->Destroy();
+		m_actionAfterWarnDialog = 0;
+	}
+	delete m_actionAfterTimer;
+	m_actionAfterTimer = 0;
+#endif
+
 	bool canQuit = true;
 	if (!SetActive(false))
 		canQuit = false;
@@ -1404,6 +1429,8 @@ void CQueueView::CheckQueueState()
 	if (m_activeMode)
 	{
 		m_activeMode = 0;
+		/* Users don't seem to like this, so comment it out for now.
+		 * maybe make it configureable in future?
 		if (!m_pQueue->GetSelection())
 		{
 			CQueueViewBase* pFailed = m_pQueue->GetQueueView_Failed();
@@ -1413,6 +1440,9 @@ void CQueueView::CheckQueueState()
 			else if (pSuccessful->GetItemCount())
 				m_pQueue->SetSelection(2);
 		}
+		*/
+		if (!m_quit)
+			ActionAfter();
 	}
 
 	if (m_quit)
@@ -1422,7 +1452,7 @@ void CQueueView::CheckQueueState()
 bool CQueueView::IncreaseErrorCount(t_EngineData& engineData)
 {
 	engineData.pItem->m_errorCount++;
-	if (engineData.pItem->m_errorCount <= COptions::Get()->GetOptionVal(OPTION_TRANSFERRETRYCOUNT))
+	if (engineData.pItem->m_errorCount <= COptions::Get()->GetOptionVal(OPTION_RECONNECTCOUNT))
 		return true;
 
 	ResetEngine(engineData, failure);
@@ -1700,18 +1730,15 @@ void CQueueView::ImportQueue(TiXmlElement* pElement, bool updateSelections)
 				wxLongLong size = GetTextElementLongLong(pFile, "Size", -1);
 				unsigned int errorCount = GetTextElementInt(pFile, "ErrorCount");
 				unsigned int priority = GetTextElementInt(pFile, "Priority", priority_normal);
-				unsigned int itemState = GetTextElementInt(pFile, "ItemState", ItemState_Wait);
 				bool binary = GetTextElementInt(pFile, "TransferMode", 1) != 0;
 
 				CServerPath remotePath;
 				if (localFile != _T("") && remoteFile != _T("") && remotePath.SetSafePath(safeRemotePath) &&
-					size >= -1 && priority < PRIORITY_COUNT &&
-					(itemState == ItemState_Wait || itemState == ItemState_Error))
+					size >= -1 && priority < PRIORITY_COUNT)
 				{
 					CFileItem* fileItem = new CFileItem(pServerItem, true, download, localFile, remoteFile, remotePath, size);
 					fileItem->m_transferSettings.binary = binary;
 					fileItem->SetPriorityRaw((enum QueuePriority)priority);
-					fileItem->SetItemState((enum ItemState)itemState);
 					fileItem->m_errorCount = errorCount;
 					InsertItem(pServerItem, fileItem);
 				}
@@ -1777,6 +1804,27 @@ void CQueueView::ImportQueue(TiXmlElement* pElement, bool updateSelections)
 
 void CQueueView::SettingsChanged()
 {
+	// Create missing engines if needed
+	const int engineCount = m_engineData.size();
+	const int newEngineCount = COptions::Get()->GetOptionVal(OPTION_NUMTRANSFERS);
+	if (newEngineCount >= engineCount)
+	{
+		for (int i = 0; i < (newEngineCount - engineCount + 1); i++)
+		{
+			t_EngineData *pData = new t_EngineData;
+			pData->active = false;
+			pData->pItem = 0;
+			pData->pStatusLineCtrl = 0;
+			pData->state = t_EngineData::none;
+			pData->m_idleDisconnectTimer = 0;
+
+			pData->pEngine = new CFileZillaEngine();
+			pData->pEngine->Init(this, COptions::Get());
+
+			m_engineData.push_back(pData);
+		}
+	}
+
 	m_asciiFiles.clear();
 	wxString extensions = COptions::Get()->GetOption(OPTION_ASCIIFILES);
 	wxString ext;
@@ -1809,6 +1857,8 @@ void CQueueView::SettingsChanged()
 	ext += extensions;
 	ext.Replace(_T("\\\\"), _T("\\"));
 	m_asciiFiles.push_back(ext);
+
+	TryStartNextTransfer();
 }
 
 bool CQueueView::ShouldUseBinaryMode(wxString filename)
@@ -1821,9 +1871,9 @@ bool CQueueView::ShouldUseBinaryMode(wxString filename)
 
 	int pos = filename.Find('.');
 	if (pos == -1)
-		return COptions::Get()->GetOptionVal(OPTION_ASCIINOEXT) != 0;
+		return COptions::Get()->GetOptionVal(OPTION_ASCIINOEXT) == 0;
 	else if (!pos)
-		return COptions::Get()->GetOptionVal(OPTION_ASCIIDOTFILE) != 0;
+		return COptions::Get()->GetOptionVal(OPTION_ASCIIDOTFILE) == 0;
 
 	wxString ext = filename;
 	do
@@ -1869,11 +1919,25 @@ void CQueueView::OnContextMenu(wxContextMenuEvent& event)
 		return;
 
 	pMenu->Check(XRCID("ID_PROCESSQUEUE"), IsActive() ? true : false);
+	pMenu->Check(XRCID("ID_ACTIONAFTER_DISABLE"), IsActionAfter(ActionAfterState_Disabled));
+	pMenu->Check(XRCID("ID_ACTIONAFTER_CLOSE"), IsActionAfter(ActionAfterState_Close));
+	pMenu->Check(XRCID("ID_ACTIONAFTER_DISCONNECT"), IsActionAfter(ActionAfterState_Disconnect));
+	pMenu->Check(XRCID("ID_ACTIONAFTER_RUNCOMMAND"), IsActionAfter(ActionAfterState_RunCommand));
+	pMenu->Check(XRCID("ID_ACTIONAFTER_SHOWMESSAGE"), IsActionAfter(ActionAfterState_ShowMessage));
+	pMenu->Check(XRCID("ID_ACTIONAFTER_PLAYSOUND"), IsActionAfter(ActionAfterState_PlaySound));
+#ifdef __WXMSW__
+	pMenu->Check(XRCID("ID_ACTIONAFTER_REBOOT"), IsActionAfter(ActionAfterState_Reboot));
+	pMenu->Check(XRCID("ID_ACTIONAFTER_SHUTDOWN"), IsActionAfter(ActionAfterState_Shutdown));
+#endif
 	pMenu->Enable(XRCID("ID_REMOVE"), GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) != -1);
 
 	const bool hasSelection = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) != -1;
+
 	pMenu->Enable(XRCID("ID_PRIORITY"), hasSelection);
 	pMenu->Enable(XRCID("ID_DEFAULT_FILEEXISTSACTION"), hasSelection);
+#ifdef __WXMSW__
+	pMenu->Enable(XRCID("ID_ACTIONAFTER"), m_actionAfterWarnDialog == NULL);
+#endif
 
 	PopupMenu(pMenu);
 	delete pMenu;
@@ -1890,6 +1954,71 @@ void CQueueView::OnStopAndClear(wxCommandEvent& event)
 	RemoveAll();
 }
 
+void CQueueView::OnActionAfter(wxCommandEvent& event)
+{
+	if (!event.IsChecked() || event.GetId() == XRCID("ID_ACTIONAFTER_DISABLE"))
+	{ // Goes from checked to non-checked or disable is pressed
+		m_actionAfterState = ActionAfterState_Disabled;
+		m_actionAfterRunCommand = _T("");
+
+#ifdef __WXMSW__
+		if (m_actionAfterWarnDialog)
+		{
+			m_actionAfterWarnDialog->Destroy();
+			m_actionAfterWarnDialog = 0;
+		}
+		delete m_actionAfterTimer;
+		m_actionAfterTimer = 0;
+#endif
+	}
+	else if (event.GetId() == XRCID("ID_ACTIONAFTER_DISABLE"))
+		m_actionAfterState = ActionAfterState_Disabled;
+
+	else if (event.GetId() == XRCID("ID_ACTIONAFTER_CLOSE"))
+		m_actionAfterState = ActionAfterState_Close;
+
+	else if (event.GetId() == XRCID("ID_ACTIONAFTER_DISCONNECT"))
+		m_actionAfterState = ActionAfterState_Disconnect;
+
+	else if (event.GetId() == XRCID("ID_ACTIONAFTER_SHOWMESSAGE"))
+		m_actionAfterState = ActionAfterState_ShowMessage;
+
+	else if (event.GetId() == XRCID("ID_ACTIONAFTER_PLAYSOUND"))
+		m_actionAfterState = ActionAfterState_PlaySound;
+
+	else if (event.GetId() == XRCID("ID_ACTIONAFTER_RUNCOMMAND"))
+	{
+		m_actionAfterState = ActionAfterState_RunCommand;
+		wxTextEntryDialog dlg(m_pMainFrame, _("Please enter a path and executable to run.\nE.g. c:\\somePath\\file.exe under MS Windows or /somePath/file under Unix.\nYou can also optionally specify program arguments."), _("Enter command"));
+
+		if (dlg.ShowModal() != wxID_OK)
+		{
+			m_actionAfterState = ActionAfterState_Disabled;
+			return;
+		}
+		const wxString &command = dlg.GetValue();
+
+		if (command == _T(""))
+		{
+			wxMessageBox(_("No command given, aborting."), _("Empty command"), wxICON_ERROR, m_pMainFrame);
+			m_actionAfterState = ActionAfterState_Disabled;
+			return;
+		}
+		m_actionAfterRunCommand = command;
+	}
+
+#ifdef __WXMSW__
+
+	else if (event.GetId() == XRCID("ID_ACTIONAFTER_REBOOT"))
+		m_actionAfterState = ActionAfterState_Reboot;
+
+	else if (event.GetId() == XRCID("ID_ACTIONAFTER_SHUTDOWN"))
+		m_actionAfterState = ActionAfterState_Shutdown;
+
+#endif
+
+}
+
 void CQueueView::RemoveAll()
 {
 	// This function removes all inactive items and queues active items
@@ -1913,6 +2042,7 @@ void CQueueView::RemoveAll()
 		}
 	}
 	SetItemCount(m_itemCount);
+	m_actionAfterState = ActionAfterState_Disabled;
 
 	m_serverList = newServerList;
 	UpdateStatusLinePositions();
@@ -2357,6 +2487,17 @@ void CQueueView::CommitChanges()
 
 void CQueueView::OnTimer(wxTimerEvent& event)
 {
+	const int id = event.GetId();
+	if (id == -1)
+		return;
+#ifdef __WXMSW__
+	if (id == m_actionAfterTimerId)
+	{
+		OnActionAfterTimerTick();
+		return;
+	}
+#endif
+
 	for (unsigned int i = 1; i < m_engineData.size(); i++)
 	{
 		t_EngineData* pData = m_engineData[i];
@@ -2487,3 +2628,130 @@ void CQueueView::OnExclusiveEngineRequestGranted(wxCommandEvent& event)
 
 	SendNextCommand(*pEngineData);
 }
+
+enum ActionAfterState CQueueView::GetActionAfterState() const
+{
+	return m_actionAfterState;
+}
+
+bool CQueueView::IsActionAfter(enum ActionAfterState state)
+{
+	return m_actionAfterState == state;
+}
+
+void CQueueView::ActionAfter(bool warned /*=false*/)
+{
+	switch (m_actionAfterState)
+	{
+		case ActionAfterState_Close:
+		{
+			m_pMainFrame->Close();
+			break;
+		}
+		case ActionAfterState_Disconnect:
+		{
+			if (m_pMainFrame->GetState()->IsRemoteConnected() && m_pMainFrame->GetState()->IsRemoteIdle())
+				m_pMainFrame->GetState()->m_pCommandQueue->ProcessCommand(new CDisconnectCommand());
+			break;
+		}
+		case ActionAfterState_RunCommand:
+		{
+			wxExecute(m_actionAfterRunCommand);
+			break;
+		}
+		case ActionAfterState_ShowMessage:
+		{
+			wxMessageDialog* dialog = new wxMessageDialog(m_pMainFrame, _T("No more files in the queue!"), _T("Queue completion"), wxOK | wxICON_INFORMATION);
+			dialog->ShowModal();
+			m_pMainFrame->RequestUserAttention(wxUSER_ATTENTION_ERROR);
+			break;
+		}
+		case ActionAfterState_PlaySound:
+		{
+			wxSound sound(wxGetApp().GetResourceDir() + _T("finished.wav"));
+			sound.Play(wxSOUND_ASYNC);
+			break;
+		}
+#ifdef __WXMSW__
+		case ActionAfterState_Reboot:
+		{
+			if (!warned)
+				ActionAfterWarnUser(_T("The system will soon reboot unless you press cancel."));
+			else
+				wxShutdown(wxSHUTDOWN_REBOOT);
+			break;
+		}
+
+		case ActionAfterState_Shutdown:
+		{
+			if (!warned)
+				ActionAfterWarnUser(_T("The system will soon shutdown unless you press cancel."));
+			else
+				wxShutdown(wxSHUTDOWN_POWEROFF);
+			break;
+		}
+#endif
+		default:
+			break;
+
+	}
+	m_actionAfterState = ActionAfterState_Disabled; // Resetting the state.
+}
+
+#ifdef __WXMSW__
+void CQueueView::ActionAfterWarnUser(wxString message)
+{
+	if (m_actionAfterWarnDialog != NULL)
+		return;
+
+	m_actionAfterWarnDialog = new wxProgressDialog(_T("Queue completion dialog"), message, 150, m_pMainFrame, wxPD_CAN_ABORT | wxPD_AUTO_HIDE | wxPD_CAN_SKIP | wxPD_APP_MODAL);
+	wxSize dialogSize = m_actionAfterWarnDialog->GetSize();
+	m_actionAfterWarnDialog->SetSize(dialogSize.GetWidth() / 2, dialogSize.GetHeight());
+	m_actionAfterWarnDialog->CentreOnParent();
+	m_actionAfterWarnDialog->SetFocus();
+	m_pMainFrame->RequestUserAttention(wxUSER_ATTENTION_ERROR);
+
+	wxASSERT(!m_actionAfterTimer);
+	m_actionAfterTimer = new wxTimer(this, m_actionAfterTimerId);
+	m_actionAfterTimerId = m_actionAfterTimer->GetId();
+	m_actionAfterTimerCount = 0;
+	m_actionAfterTimer->Start(100, wxTIMER_CONTINUOUS);
+}
+
+void CQueueView::OnActionAfterTimerTick()
+{
+	if (!m_actionAfterWarnDialog)
+	{
+		delete m_actionAfterTimer;
+		m_actionAfterTimer = 0;
+		return;
+	}
+
+	bool skipped = false;
+	if (m_actionAfterTimerCount > 150)
+	{
+		m_actionAfterWarnDialog->Destroy();
+		m_actionAfterWarnDialog = 0;
+		delete m_actionAfterTimer;
+		m_actionAfterTimer = 0;
+		ActionAfter(true);
+	}
+	else if (!m_actionAfterWarnDialog->Update(m_actionAfterTimerCount++, _T(""), &skipped))
+	{
+		// User has pressed cancel!
+		m_actionAfterState = ActionAfterState_Disabled; // resetting to disabled
+		m_actionAfterWarnDialog->Destroy();
+		m_actionAfterWarnDialog = 0;
+		delete m_actionAfterTimer;
+		m_actionAfterTimer = 0;
+	}
+	else if (skipped)
+	{
+		m_actionAfterWarnDialog->Destroy();
+		m_actionAfterWarnDialog = 0;
+		delete m_actionAfterTimer;
+		m_actionAfterTimer = 0;
+		ActionAfter(true);
+	}
+}
+#endif //__WXMSW__
