@@ -13,7 +13,11 @@ public:
 	enum DialogType
 	{
 		rawcommand_quote,
-		viewhidden
+		viewhidden,
+		confirmexit,
+		sitemanager_confirmdelete,
+		confirmexit_edit, // Confirm closing FileZilla while files are still being edited
+		confirm_preserve_timestamps
 	};
 
 	CConditionalDialog(wxWindow* parent, enum DialogType type, enum Modes mode, bool checked = false);
