@@ -1,6 +1,9 @@
 #include "FileZilla.h"
 #include "fileexistsdlg.h"
 
+#include <wx/display.h>
+#include <wx/string.h>
+
 BEGIN_EVENT_TABLE(CFileExistsDlg, wxDialogEx)
 EVT_BUTTON(XRCID("wxID_OK"), CFileExistsDlg::OnOK)
 EVT_BUTTON(XRCID("wxID_CANCEL"), CFileExistsDlg::OnCancel)
@@ -23,12 +26,12 @@ bool CFileExistsDlg::Create(wxWindow* parent)
 	CreateControls();
 	GetSizer()->Fit(this);
 	GetSizer()->SetSizeHints(this);
-	
-	return TRUE;
+
+	return true;
 }
 
 void CFileExistsDlg::CreateControls()
-{	
+{
 	wxXmlResource::Get()->LoadDialog(this, GetParent(), _T("ID_FILEEXISTSDLG"));
 	m_pAction1 = wxDynamicCast(FindWindow(XRCID("ID_ACTION1")), wxRadioButton);
 	m_pAction2 = wxDynamicCast(FindWindow(XRCID("ID_ACTION2")), wxRadioButton);
@@ -37,19 +40,22 @@ void CFileExistsDlg::CreateControls()
 	m_pAction5 = wxDynamicCast(FindWindow(XRCID("ID_ACTION5")), wxRadioButton);
 
 	wxString localFile = m_pNotification->localFile;
-	localFile.Replace(_T("&"), _T("&&"));
 
 	wxString remoteFile = m_pNotification->remotePath.GetPath() + m_pNotification->remoteFile;
+    localFile = GetPathEllipsis(localFile, FindWindow(XRCID("ID_FILE1_NAME")));
+    remoteFile = GetPathEllipsis(remoteFile, FindWindow(XRCID("ID_FILE2_NAME")));
+
+	localFile.Replace(_T("&"), _T("&&"));
 	remoteFile.Replace(_T("&"), _T("&&"));
 
 	if (m_pNotification->download)
 	{
 		wxStaticText *pStatText;
-		
+
 		pStatText = reinterpret_cast<wxStaticText *>(FindWindow(XRCID("ID_FILE1_NAME")));
 		if (pStatText)
 			pStatText->SetLabel(localFile);
-		
+
 		pStatText = reinterpret_cast<wxStaticText *>(FindWindow(XRCID("ID_FILE1_SIZE")));
 		if (pStatText)
 		{
@@ -58,7 +64,7 @@ void CFileExistsDlg::CreateControls()
 			else
 				pStatText->SetLabel(_("Size unknown"));
 		}
-		
+
 		pStatText = reinterpret_cast<wxStaticText *>(FindWindow(XRCID("ID_FILE1_TIME")));
 		if (pStatText)
 		{
@@ -73,7 +79,7 @@ void CFileExistsDlg::CreateControls()
 		pStatText = reinterpret_cast<wxStaticText *>(FindWindow(XRCID("ID_FILE2_NAME")));
 		if (pStatText)
 			pStatText->SetLabel(remoteFile);
-		
+
 		pStatText = reinterpret_cast<wxStaticText *>(FindWindow(XRCID("ID_FILE2_SIZE")));
 		if (pStatText)
 		{
@@ -82,7 +88,7 @@ void CFileExistsDlg::CreateControls()
 			else
 				pStatText->SetLabel(_("Size unknown"));
 		}
-		
+
 		pStatText = reinterpret_cast<wxStaticText *>(FindWindow(XRCID("ID_FILE2_TIME")));
 		if (pStatText)
 		{
@@ -105,7 +111,7 @@ void CFileExistsDlg::CreateControls()
 		pStatText = reinterpret_cast<wxStaticText *>(FindWindow(XRCID("ID_FILE1_NAME")));
 		if (pStatText)
 			pStatText->SetLabel(remoteFile);
-		
+
 		pStatText = reinterpret_cast<wxStaticText *>(FindWindow(XRCID("ID_FILE1_SIZE")));
 		if (pStatText)
 		{
@@ -161,7 +167,7 @@ void CFileExistsDlg::LoadIcon(int id, const wxString &file)
 	wxStaticBitmap *pStatBmp = reinterpret_cast<wxStaticBitmap *>(FindWindow(id));
 	if (!pStatBmp)
 		return;
-	
+
 #ifdef __WXMSW__
 	SHFILEINFO fileinfo;
 	memset(&fileinfo,0,sizeof(fileinfo));
@@ -256,7 +262,7 @@ void CFileExistsDlg::OnOK(wxCommandEvent& event)
 		m_action = 4;
 	else
 		m_action = 0;
-	
+
 	m_always = XRCCTRL(*this, "ID_ALWAYS", wxCheckBox)->GetValue();
 	m_directionOnly = XRCCTRL(*this, "ID_UPDOWNONLY", wxCheckBox)->GetValue();
 	m_queueOnly = XRCCTRL(*this, "ID_QUEUEONLY", wxCheckBox)->GetValue();
@@ -280,3 +286,59 @@ bool CFileExistsDlg::Always(bool &directionOnly, bool &queueOnly) const
 	queueOnly = m_queueOnly;
 	return m_always;
 }
+
+wxString CFileExistsDlg::GetPathEllipsis(wxString path, wxWindow *window)
+{
+	int string_width; // width of the path string in pixels
+	int y;            // dummy variable
+	window->GetTextExtent(path, &string_width, &y);
+
+	wxDisplay display(wxDisplay::GetFromWindow(window));
+	wxRect rect = display.GetClientArea();
+	const int DESKTOP_WIDTH = rect.GetWidth(); // width of the desktop in pixels
+	const int maxWidth = DESKTOP_WIDTH * 0.75;
+
+	// If the path is already short enough, don't change it
+	if (string_width <= maxWidth || path.Length() < 20)
+		return path;
+
+	wxString fill = _T(" ");
+#if wxUSE_UNICODE
+	fill += 0x2026; //unicode ellipsis character
+#else
+	fill += _T("...");
+#endif
+	fill += _T(" ");
+
+	int fillWidth;
+	window->GetTextExtent(fill, &fillWidth, &y);
+
+	// Do initial split roughly in the middle of the string
+	int middle = path.Length() / 2;
+	wxString left = path.Left(middle);
+	wxString right = path.Mid(middle);
+
+	int leftWidth, rightWidth;
+	window->GetTextExtent(left, &leftWidth, &y);
+	window->GetTextExtent(right, &rightWidth, &y);
+
+	// continue removing one character at a time around the fill until path string is small enough
+	while ((leftWidth + fillWidth + rightWidth) > maxWidth)
+	{
+		if (leftWidth > rightWidth && left.Len() > 10)
+		{
+			left.RemoveLast();
+			window->GetTextExtent(left, &leftWidth, &y);
+		}
+		else
+		{
+			if (right.Len() <= 10)
+				break;
+
+			right = right.Mid(1);
+			window->GetTextExtent(right, &rightWidth, &y);
+		}
+	}
+
+	return left + fill + right;
+}
